@@ -1,47 +1,46 @@
-"""
-The `account` command group.
-"""
-
 import getpass
 import json
-from typing import Dict, List, Optional
+from typing import List, Optional
 
+import click
 import eth_account
 from autonity import Autonity
-from click import ClickException, Path, argument, command, group, option
+from click import ClickException, Path, argument, group, option
 from eth_account import Account
 from eth_account.messages import encode_defunct
 from hexbytes import HexBytes
 from web3 import Web3
 from web3.types import BlockIdentifier
 
-from .. import config
-from ..erc20 import ERC20
-from ..logging import log
+from .. import config, device
+from ..auth import (
+    validate_authenticator,
+    validate_authenticator_account,
+)
 from ..denominations import (
     format_auton_quantity,
     format_newton_quantity,
     format_quantity,
 )
+from ..erc20 import ERC20
 from ..keyfile import (
     PrivateKey,
     create_keyfile_from_private_key,
-    decrypt_keyfile,
     get_address_from_keyfile,
 )
+from ..logging import log
 from ..options import (
-    from_option,
-    keyfile_and_password_options,
+    authentication_options,
+    from_options,
     keyfile_option,
     keystore_option,
     newton_or_token_option,
+    optgroup,
     rpc_endpoint_option,
 )
-from ..tx import sign_tx
 from ..user import get_account_stats
 from ..utils import (
     address_keyfile_dict,
-    from_address_from_argument_optional,
     load_from_file_or_stdin,
     load_from_file_or_stdin_line,
     new_keyfile_from_options,
@@ -51,8 +50,6 @@ from ..utils import (
     web3_from_endpoint_arg,
 )
 
-# pylint: disable=too-many-locals
-
 
 @group(name="account")
 def account_group() -> None:
@@ -61,29 +58,56 @@ def account_group() -> None:
     """
 
 
-@command(name="list")
-@option("--with-files", is_flag=True, help="also show keyfile names.")
-@keystore_option()
-def list_cmd(keystore: Optional[str], with_files: bool) -> None:
+@account_group.command(name="list")
+@optgroup.group("Keyfile accounts")
+@keystore_option(cls=optgroup.option)
+@optgroup.group("Trezor accounts")
+@optgroup.option("--trezor", is_flag=True, help="Enumerate Trezor accounts")
+@optgroup.option(
+    "--prefix",
+    metavar="PREFIX",
+    default=device.TREZOR_DEFAULT_PREFIX,
+    show_default=True,
+    help="Custom BIP32 derivation prefix",
+)
+@optgroup.option(
+    "--start",
+    type=int,
+    default=0,
+    show_default=True,
+    help="Start index at BIP32 derivation prefix",
+)
+@optgroup.option(
+    "-n",
+    type=int,
+    default=20,
+    show_default=True,
+    help="Number of Trezor accounts to list",
+)
+def list_cmd(
+    keystore: Optional[str], trezor: bool, prefix: str, start: int, n: int
+) -> None:
     """
-    List the accounts for files in the keystore directory.
+    List accounts in keyfiles or in a Trezor device.
     """
 
-    keystore = config.get_keystore_directory(keystore)
-    keyfiles = address_keyfile_dict(keystore)
-    for addr, keyfile in keyfiles.items():
-        if with_files:
-            print(addr + " " + keyfile)
-        else:
-            print(addr)
+    if trezor and keystore:
+        raise click.ClickException(
+            "Options --trezor and --keystore are mutually exclusive."
+        )
+
+    if trezor:
+        accounts = device.enumerate_accounts(prefix, start, n)
+    else:
+        keystore = config.get_keystore_directory(keystore)
+        accounts = address_keyfile_dict(keystore).items()
+    for addr, path in accounts:
+        print(addr + " " + path)
 
 
-account_group.add_command(list_cmd)
-
-
-@command()
+@account_group.command()
 @rpc_endpoint_option
-@keyfile_option()
+@authentication_options()
 @option(
     "--asof",
     help="state as of TAG, one of block number, 'latest', 'earliest', or 'pending'.",
@@ -92,20 +116,19 @@ account_group.add_command(list_cmd)
 def info(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     accounts: List[str],
     asof: Optional[BlockIdentifier],
 ) -> None:
     """
-    Print some information about the given account.
+    Print information about the given account.
 
-    Falls back to the default keyfile account if no accounts specified.
+    Falls back to the default keyfile if no account is specified.
     """
 
     if len(accounts) == 0:
-        account = from_address_from_argument_optional(None, keyfile)
-        if not account:
-            raise ClickException("No accounts specified")
-        accounts = [account]
+        auth = validate_authenticator(keyfile=keyfile, trezor=trezor)
+        accounts = [auth.address]
 
     addresses = [Web3.to_checksum_address(act) for act in accounts]
 
@@ -114,18 +137,16 @@ def info(
     print(to_json(account_stats, pretty=True))
 
 
-account_group.add_command(info)
-
-
-@command()
+@account_group.command()
 @rpc_endpoint_option
 @newton_or_token_option
-@keyfile_option()
+@authentication_options()
 @argument("account_str", metavar="ACCOUNT", default="")
 def balance(
     rpc_endpoint: Optional[str],
     account_str: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     ntn: bool,
     token: Optional[str],
 ) -> None:
@@ -133,11 +154,9 @@ def balance(
     Print the current balance of the given account.
     """
 
-    account_addr = from_address_from_argument_optional(account_str, keyfile)
-    if not account_addr:
-        raise ClickException(
-            "could not determine account address from argument or keyfile"
-        )
+    account_addr = validate_authenticator_account(
+        account_str, keyfile=keyfile, trezor=trezor
+    )
 
     token_addresss = newton_or_token_to_address(ntn, token)
 
@@ -160,63 +179,18 @@ def balance(
         print(format_auton_quantity(w3.eth.get_balance(account_addr)))
 
 
-account_group.add_command(balance)
-
-
-@command()
-@rpc_endpoint_option
-@keyfile_option()
-@argument("account_str", metavar="ACCOUNT", default="")
-def lntn_balances(
-    rpc_endpoint: Optional[str], account_str: Optional[str], keyfile: Optional[str]
-) -> None:
-    """
-    Print all Liquid Newton balances of the given account.
-    """
-
-    account_addr = from_address_from_argument_optional(account_str, keyfile)
-    if not account_addr:
-        raise ClickException(
-            "could not determine account address from argument or keyfile"
-        )
-
-    w3 = web3_from_endpoint_arg(None, rpc_endpoint)
-    aut = Autonity(w3)
-    validator_addrs = aut.get_validators()
-    validators = [aut.get_validator(vaddr) for vaddr in validator_addrs]
-
-    balances: Dict[str, str] = {}
-    for validator in validators:
-        log(f"computing holdings for validators {validator.node_address}")
-        lntn = ERC20(w3, validator.liquid_state_contract)
-        bal = lntn.balance_of(account_addr)
-        if bal:
-            balances[validator.node_address] = format_newton_quantity(bal)
-
-    print(to_json(balances, pretty=True))
-
-
-account_group.add_command(lntn_balances)
-
-
-@command()
+@account_group.command()
 @keystore_option()
-@keyfile_option(required=False, output=True)
+@keyfile_option(output=True)
 @option(
     "--extra-entropy",
     type=Path(),
-    help="File containing extra entropy.  Use '-' to prompt for keyboard input.",
-)
-@option(
-    "--show-password",
-    is_flag=True,
-    help="Echo password input to the terminal",
+    help="file containing extra entropy. Use '-' to prompt for keyboard input.",
 )
 def new(
     keystore: Optional[str],
     keyfile: Optional[str],
     extra_entropy: Optional[str],
-    show_password: bool,
 ) -> None:
     """
     Create a new key and write it to a keyfile.
@@ -239,14 +213,14 @@ def new(
     # Ask for password (and confirmation) and ensure both entries
     # match.
 
-    password = prompt_for_new_password(show_password)
+    password = prompt_for_new_password()
     log("Generating private key ...")
     account = eth_account.Account.create(entropy)
     keyfile_data = create_keyfile_from_private_key(account.key, password)
     keyfile_addr = get_address_from_keyfile(keyfile_data)
     if account.address != keyfile_addr:
         raise ClickException(
-            f"internal error (address-mismatch) {account.address} != {keyfile_addr}"
+            f"Internal error (address-mismatch) {account.address} != {keyfile_addr}"
         )
 
     # If keyfile was not given, generate a new keyfile based on
@@ -260,37 +234,28 @@ def new(
     print(f"{keyfile_addr}  {keyfile}")
 
 
-account_group.add_command(new)
-
-
-@command()
+@account_group.command()
 @keystore_option()
 @keyfile_option(output=True)
-@option(
-    "--show-password",
-    is_flag=True,
-    help="Echo password input to the terminal",
-)
 @argument("private_key_file", type=Path(exists=False))
 def import_private_key(
     keystore: Optional[str],
     keyfile: Optional[str],
-    show_password: bool,
     private_key_file: str,
 ) -> None:
     """
-    Read a plaintext private key file (as hex), and create a new encrypted keystore
-    file for it.
+    Read a plaintext private key file (as hex-string), and create a new encrypted
+    keystore file for it.
 
-    Use - to read private key from stdin. If no keyfile is specified, a default name
-    is used (consistent with GETH keyfiles) in the keystore.
+    Use '-' to read private key from standard input. If no keyfile is specified,
+    a default name is used (consistent with GETH keyfiles) in the keystore.
     """
 
     private_key = HexBytes.fromhex(load_from_file_or_stdin_line(private_key_file))
     if len(private_key) != 32:
-        raise ClickException("invalid private key length")
+        raise ClickException("Invalid private key length")
 
-    password = prompt_for_new_password(show_password)
+    password = prompt_for_new_password()
 
     keyfile_data = create_keyfile_from_private_key(PrivateKey(private_key), password)
     keyfile_addr = get_address_from_keyfile(keyfile_data)
@@ -305,10 +270,7 @@ def import_private_key(
     print(f"{keyfile_addr}  {keyfile}")
 
 
-account_group.add_command(import_private_key)
-
-
-@command()
+@account_group.command()
 @argument("keyfile-path", required=False, type=Path(exists=True))
 def reveal_private_key(
     keyfile_path: Optional[str],
@@ -337,57 +299,51 @@ def reveal_private_key(
     except ValueError as e:
         raise ClickException(str(e))
 
-    print(key.hex())
+    print(Web3.to_hex(key))
 
 
-account_group.add_command(reveal_private_key)
-
-
-@command()
-@keyfile_and_password_options()
+@account_group.command()
+@authentication_options()
 @argument(
     "tx-file",
     type=Path(),
     required=True,
 )
-def signtx(keyfile: Optional[str], password: Optional[str], tx_file: str) -> None:
+def signtx(keyfile: Optional[str], trezor: Optional[str], tx_file: str) -> None:
     """
     Sign a transaction using the given keyfile.
 
-    Use '-' to read from stdin instead of a file.
+    Use '-' to read from standard input instead of a file.
 
-    If password is not given, the env variable 'KEYFILEPWD' is used.
-    If that is not set, the user is prompted.
+    If the environment variable 'KEYFILEPWD' is set, this keyfile password will
+    be used used. If this is not set, the user is prompted.
     """
 
     # Read tx
     tx = json.loads(load_from_file_or_stdin(tx_file))
 
-    # Read keyfile
-    keyfile = config.get_keyfile(keyfile)
-    log(f"using key file: {keyfile}")
-    with open(keyfile, encoding="ascii") as key_f:
-        encrypted_key = json.load(key_f)
+    # Get auth
+    auth = validate_authenticator(keyfile=keyfile, trezor=trezor)
 
-    # Read password
-    password = config.get_keyfile_password(password)
+    # Check for mismatch
+    if "from" in tx and tx["from"] != auth.address:
+        raise ClickException(
+            f"TX from-address {tx['from']} does not match signer's address {auth.address}."
+        )
 
     # Sign the tx:
-    signed_tx = sign_tx(tx, encrypted_key, password)
+    signed_tx = auth.sign_transaction(tx)
 
     print(to_json(signed_tx._asdict()))
 
 
-account_group.add_command(signtx)
-
-
-@command()
-@keyfile_and_password_options()
+@account_group.command()
+@authentication_options()
 @option(
     "--use-message-file",
     "-f",
     is_flag=True,
-    help="Interpret MESSAGE as a filename where - means stdin",
+    help="interpret MESSAGE as a filename where '-' means standard input.",
 )
 @argument(
     "message",
@@ -396,7 +352,7 @@ account_group.add_command(signtx)
 @argument("signature-file", type=Path(), required=False)
 def sign_message(
     keyfile: Optional[str],
-    password: Optional[str],
+    trezor: Optional[str],
     use_message_file: bool,
     message: str,
     signature_file: Optional[str],
@@ -405,7 +361,7 @@ def sign_message(
     Use the private key in the given keyfile to sign the string
     MESSAGE (or the contents of a file; see --use-message-file).
 
-    The signature is always written to stdout (which can be piped to a
+    The signature is always written to standard output (which can be piped to a
     file). The signature is also written to SIGNATURE_FILE, if given.
     """
 
@@ -413,21 +369,12 @@ def sign_message(
     if use_message_file:
         message = load_from_file_or_stdin(message)
 
-    # Read keyfile
-    keyfile = config.get_keyfile(keyfile)
-    log(f"using key file: {keyfile}")
-    with open(keyfile, encoding="ascii") as key_f:
-        encrypted_key = json.load(key_f)
-
-    # Read password
-    password = config.get_keyfile_password(password)
-    private_key = decrypt_keyfile(encrypted_key, password)
+    # Get auth
+    auth = validate_authenticator(keyfile=keyfile, trezor=trezor)
 
     # Sign the message
-    signature_data = Account().sign_message(
-        signable_message=encode_defunct(text=message), private_key=private_key
-    )
-    signature = signature_data["signature"].hex()
+    log(f'Signing message: "{message}" (len={len(message)})')
+    signature = auth.sign_message(message).hex()
 
     # Optionally write to the output file
     if signature_file:
@@ -437,17 +384,13 @@ def sign_message(
     print(signature)
 
 
-account_group.add_command(sign_message)
-
-
-@command()
-@keyfile_option()
-@from_option
+@account_group.command()
+@from_options()
 @option(
     "--use-message-file",
     "-m",
     is_flag=True,
-    help="Interpret MESSAGE as a filename where - means stdin",
+    help="interpret MESSAGE as a filename where '-' means standard input.",
 )
 @argument(
     "message",
@@ -460,8 +403,9 @@ account_group.add_command(sign_message)
     required=True,
 )
 def verify_signature(
-    keyfile: Optional[str],
     from_str: Optional[str],
+    keyfile: Optional[str],
+    trezor: Optional[str],
     use_message_file: bool,
     message: str,
     signature_file: str,
@@ -481,7 +425,7 @@ def verify_signature(
         signature_hex = signature_f.read().rstrip()
         signature = HexBytes(signature_hex)
 
-    from_addr = from_address_from_argument_optional(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
 
     recovered_addr = Account().recover_message(
         encode_defunct(text=message), signature=signature
@@ -492,6 +436,3 @@ def verify_signature(
         raise ClickException("Signature invalid")
 
     log("signature is valid")
-
-
-account_group.add_command(verify_signature)
