@@ -1,10 +1,6 @@
-"""
-The "block" command group
-"""
-
 from typing import Optional
 
-from click import argument, command, group
+from click import argument, group
 
 from ..options import rpc_endpoint_option
 from ..user import get_block
@@ -18,12 +14,12 @@ def block_group() -> None:
     """
 
 
-@command()
+@block_group.command()
 @rpc_endpoint_option
 @argument("identifier", default="latest")
 def get(rpc_endpoint: Optional[str], identifier: str) -> None:
     """
-    Print information for block.
+    Print information about the given block.
 
     <identifier> is a block number or hash. If no argument is given, "latest" is used.
     """
@@ -34,10 +30,7 @@ def get(rpc_endpoint: Optional[str], identifier: str) -> None:
     print(to_json(block_data))
 
 
-block_group.add_command(get)
-
-
-@command()
+@block_group.command()
 @rpc_endpoint_option
 def height(rpc_endpoint: Optional[str]) -> None:
     """
@@ -46,6 +39,3 @@ def height(rpc_endpoint: Optional[str]) -> None:
 
     w3 = web3_from_endpoint_arg(None, rpc_endpoint)
     print(w3.eth.block_number)
-
-
-block_group.add_command(height)
