@@ -1,10 +1,6 @@
-"""
-Implementation of `node` subcommands
-"""
-
 from typing import Optional, Union
 
-from click import command, group
+from click import group
 from web3.datastructures import AttributeDict
 from web3.exceptions import MethodUnavailable
 from web3.types import SyncStatus
@@ -20,7 +16,7 @@ def node_group() -> None:
     """
 
 
-@command()
+@node_group.command()
 @rpc_endpoint_option
 def info(rpc_endpoint: Optional[str]) -> None:
     """
@@ -38,7 +34,7 @@ def info(rpc_endpoint: Optional[str]) -> None:
 
     # Handle eth_syncing as an AttributeDict or boolean
     syncing: Union[SyncStatus, bool] = w3.eth.syncing
-    eth_syncing: Union[dict, bool] = (
+    eth_syncing: Union[dict[str, int], bool] = (
         dict(syncing) if isinstance(syncing, AttributeDict) else False
     )
     node_info = {
@@ -62,6 +58,3 @@ def info(rpc_endpoint: Optional[str]) -> None:
         )
 
     print(to_json(node_info, pretty=True))
-
-
-node_group.add_command(info)
