@@ -1,21 +1,14 @@
-"""
-The `protocol` command group.
-"""
-
 from typing import Any, Optional, Sequence
 
 from autonity import Autonity
 from autonity.constants import AUTONITY_CONTRACT_ADDRESS
 from autonity.contracts.autonity import ABI
-from click import argument, command, group
+from click import argument, group
 from web3 import Web3
 
 from ..options import rpc_endpoint_option
 from ..utils import autonity_from_endpoint_arg, to_json, web3_from_endpoint_arg
 
-# pylint: disable=too-many-arguments
-# pylint: disable=too-many-locals
-
 
 @group(name="protocol")
 def protocol_group() -> None:
@@ -34,98 +27,88 @@ def _show_json(value: Any) -> str:
     return to_json(value, pretty=True)
 
 
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def config(rpc_endpoint: Optional[str]) -> None:
     """
-    Print the Autonity contract config.
+    The Autonity contract configuration.
     """
 
-    print(_show_json(autonity_from_endpoint_arg(rpc_endpoint).config()))
+    print(_show_json(autonity_from_endpoint_arg(rpc_endpoint).get_config()))
 
 
-protocol_group.add_command(config)
+@protocol_group.command()
+@rpc_endpoint_option
+def client_config(rpc_endpoint: Optional[str]) -> None:
+    """
+    The current client-aware configuration.
+    """
+
+    print(_show_json(autonity_from_endpoint_arg(rpc_endpoint).get_client_config()))
 
 
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def epoch_id(rpc_endpoint: Optional[str]) -> None:
     """
     ID of the current epoch.
     """
 
-    print(autonity_from_endpoint_arg(rpc_endpoint).epoch_id())
+    print(autonity_from_endpoint_arg(rpc_endpoint).get_epoch_id())
 
 
-protocol_group.add_command(epoch_id)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def last_epoch_time(rpc_endpoint: Optional[str]) -> None:
     """
     Timestamp of the last epoch.
     """
 
-    print(autonity_from_endpoint_arg(rpc_endpoint).last_epoch_time())
+    print(autonity_from_endpoint_arg(rpc_endpoint).get_last_epoch_time())
 
 
-protocol_group.add_command(last_epoch_time)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def epoch_total_bonded_stake(rpc_endpoint: Optional[str]) -> None:
     """
     Total stake bonded this epoch.
     """
 
-    print(autonity_from_endpoint_arg(rpc_endpoint).epoch_total_bonded_stake())
+    print(autonity_from_endpoint_arg(rpc_endpoint).get_epoch_total_bonded_stake())
 
 
-protocol_group.add_command(epoch_total_bonded_stake)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def inflation_reserve(rpc_endpoint: Optional[str]) -> None:
     """
     The inflation reserve.
     """
 
-    print(autonity_from_endpoint_arg(rpc_endpoint).inflation_reserve())
+    print(autonity_from_endpoint_arg(rpc_endpoint).get_inflation_reserve())
 
 
-protocol_group.add_command(inflation_reserve)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
-def deployer(rpc_endpoint: Optional[str]) -> None:
+def current_epoch_period(rpc_endpoint: Optional[str]) -> None:
     """
-    Contract deployer.
+    The current epoch period in blocks.
     """
 
-    print(autonity_from_endpoint_arg(rpc_endpoint).deployer())
+    print(autonity_from_endpoint_arg(rpc_endpoint).get_current_epoch_period())
 
 
-protocol_group.add_command(deployer)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def epoch_period(rpc_endpoint: Optional[str]) -> None:
     """
-    Epoch period in blocks.
+    The current epoch period in blocks. If there is an update at epoch end,
+    the new epoch period is returned.
     """
 
     print(autonity_from_endpoint_arg(rpc_endpoint).get_epoch_period())
 
 
-protocol_group.add_command(epoch_period)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def block_period(rpc_endpoint: Optional[str]) -> None:
     """
@@ -135,10 +118,7 @@ def block_period(rpc_endpoint: Optional[str]) -> None:
     print(autonity_from_endpoint_arg(rpc_endpoint).get_block_period())
 
 
-protocol_group.add_command(block_period)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def unbonding_period(rpc_endpoint: Optional[str]) -> None:
     """
@@ -148,10 +128,7 @@ def unbonding_period(rpc_endpoint: Optional[str]) -> None:
     print(autonity_from_endpoint_arg(rpc_endpoint).get_unbonding_period())
 
 
-protocol_group.add_command(unbonding_period)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def last_epoch_block(rpc_endpoint: Optional[str]) -> None:
     """
@@ -161,10 +138,7 @@ def last_epoch_block(rpc_endpoint: Optional[str]) -> None:
     print(autonity_from_endpoint_arg(rpc_endpoint).get_last_epoch_block())
 
 
-protocol_group.add_command(last_epoch_block)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def version(rpc_endpoint: Optional[str]) -> None:
     """
@@ -174,10 +148,7 @@ def version(rpc_endpoint: Optional[str]) -> None:
     print(autonity_from_endpoint_arg(rpc_endpoint).get_version())
 
 
-protocol_group.add_command(version)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def epoch_info(rpc_endpoint: Optional[str]):
     """
@@ -188,36 +159,27 @@ def epoch_info(rpc_endpoint: Optional[str]):
     print(_show_json(aut.get_epoch_info()))
 
 
-protocol_group.add_command(epoch_info)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def committee(rpc_endpoint: Optional[str]) -> None:
     """
-    Get current committee.
+    The current committee.
     """
 
     print(_show_json(autonity_from_endpoint_arg(rpc_endpoint).get_committee()))
 
 
-protocol_group.add_command(committee)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def validators(rpc_endpoint: Optional[str]) -> None:
     """
-    Get current validators.
+    The current validators.
     """
 
     print(_show_sequence(autonity_from_endpoint_arg(rpc_endpoint).get_validators()))
 
 
-protocol_group.add_command(validators)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def treasury_account(rpc_endpoint: Optional[str]) -> None:
     """
@@ -227,10 +189,7 @@ def treasury_account(rpc_endpoint: Optional[str]) -> None:
     print(autonity_from_endpoint_arg(rpc_endpoint).get_treasury_account())
 
 
-protocol_group.add_command(treasury_account)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def treasury_fee(rpc_endpoint: Optional[str]) -> None:
     """
@@ -240,10 +199,17 @@ def treasury_fee(rpc_endpoint: Optional[str]) -> None:
     print(autonity_from_endpoint_arg(rpc_endpoint).get_treasury_fee())
 
 
-protocol_group.add_command(treasury_fee)
+@protocol_group.command()
+@rpc_endpoint_option
+def current_committee_size(rpc_endpoint: Optional[str]) -> None:
+    """
+    Current committee size.
+    """
+
+    print(autonity_from_endpoint_arg(rpc_endpoint).get_current_committee_size())
 
 
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def max_committee_size(rpc_endpoint: Optional[str]) -> None:
     """
@@ -253,10 +219,7 @@ def max_committee_size(rpc_endpoint: Optional[str]) -> None:
     print(autonity_from_endpoint_arg(rpc_endpoint).get_max_committee_size())
 
 
-protocol_group.add_command(max_committee_size)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def committee_enodes(rpc_endpoint: Optional[str]) -> None:
     """
@@ -266,10 +229,7 @@ def committee_enodes(rpc_endpoint: Optional[str]) -> None:
     print(autonity_from_endpoint_arg(rpc_endpoint).get_committee_enodes())
 
 
-protocol_group.add_command(committee_enodes)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def minimum_base_fee(rpc_endpoint: Optional[str]) -> None:
     """
@@ -279,24 +239,18 @@ def minimum_base_fee(rpc_endpoint: Optional[str]) -> None:
     print(autonity_from_endpoint_arg(rpc_endpoint).get_minimum_base_fee())
 
 
-protocol_group.add_command(minimum_base_fee)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def max_schedule_duration(rpc_endpoint: Optional[str]) -> None:
     """
-    The max allowed duration of any schedule or contract.
+    The maximum allowed duration of any schedule or contract.
     """
 
     aut = Autonity(web3_from_endpoint_arg(None, rpc_endpoint))
     print(aut.get_max_schedule_duration())
 
 
-protocol_group.add_command(max_schedule_duration)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 def operator(rpc_endpoint: Optional[str]) -> None:
     """
@@ -306,10 +260,7 @@ def operator(rpc_endpoint: Optional[str]) -> None:
     print(autonity_from_endpoint_arg(rpc_endpoint).get_operator())
 
 
-protocol_group.add_command(operator)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 @argument("block-height", type=int)
 def epoch_by_height(rpc_endpoint: Optional[str], block_height: int):
@@ -321,10 +272,7 @@ def epoch_by_height(rpc_endpoint: Optional[str], block_height: int):
     print(_show_json(aut.get_epoch_by_height(block_height)))
 
 
-protocol_group.add_command(epoch_by_height)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 @argument("block", type=int)
 def epoch_from_block(rpc_endpoint: Optional[str], block: int) -> None:
@@ -336,10 +284,7 @@ def epoch_from_block(rpc_endpoint: Optional[str], block: int) -> None:
     print(aut.get_epoch_from_block(block))
 
 
-protocol_group.add_command(epoch_from_block)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 @argument("unbonding-id", type=int)
 def is_unbonding_released(rpc_endpoint: Optional[str], unbonding_id: int):
@@ -353,10 +298,7 @@ def is_unbonding_released(rpc_endpoint: Optional[str], unbonding_id: int):
     print(int(aut.is_unbonding_released(unbonding_id)))
 
 
-protocol_group.add_command(is_unbonding_released)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 @argument("unbonding-id", type=int)
 def unbonding_share(rpc_endpoint: Optional[str], unbonding_id: int):
@@ -368,10 +310,7 @@ def unbonding_share(rpc_endpoint: Optional[str], unbonding_id: int):
     print(aut.get_unbonding_share(unbonding_id))
 
 
-protocol_group.add_command(unbonding_share)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 @argument("vault-address-str", metavar="VAULT-ADDRESS")
 @argument("index", type=int)
@@ -385,10 +324,7 @@ def schedule(rpc_endpoint: Optional[str], vault_address_str: str, index: int):
     print(_show_json(aut.get_schedule(vault_address, index)))
 
 
-protocol_group.add_command(schedule)
-
-
-@command()
+@protocol_group.command()
 @rpc_endpoint_option
 @argument("vault-address-str", metavar="VAULT-ADDRESS")
 def total_schedules(rpc_endpoint: Optional[str], vault_address_str: str):
@@ -401,10 +337,35 @@ def total_schedules(rpc_endpoint: Optional[str], vault_address_str: str):
     print(aut.get_total_schedules(vault_address))
 
 
-protocol_group.add_command(total_schedules)
+@protocol_group.command()
+@rpc_endpoint_option
+@argument("id", type=int)
+def bonding_request(rpc_endpoint: Optional[str], id: int):
+    """
+    The bonding request corresponding to the given bonding ID.
+    """
+
+    aut = Autonity(web3_from_endpoint_arg(None, rpc_endpoint))
+    bonding_request = aut.get_bonding_request_by_id(id)
+    if bonding_request:
+        print(_show_json(bonding_request))
 
 
-@command()
+@protocol_group.command()
+@rpc_endpoint_option
+@argument("id", type=int)
+def unbonding_request(rpc_endpoint: Optional[str], id: int):
+    """
+    The unbonding request corresponding to the given unbonding ID.
+    """
+
+    aut = Autonity(web3_from_endpoint_arg(None, rpc_endpoint))
+    unbonding_request = aut.get_unbonding_request_by_id(id)
+    if unbonding_request:
+        print(_show_json(unbonding_request))
+
+
+@protocol_group.command()
 def contract_address() -> None:
     """
     Address of the Autonity Contract.
@@ -413,16 +374,10 @@ def contract_address() -> None:
     print(AUTONITY_CONTRACT_ADDRESS)
 
 
-protocol_group.add_command(contract_address)
-
-
-@command()
+@protocol_group.command()
 def contract_abi() -> None:
     """
     ABI of the Autonity Contract.
     """
 
     print(_show_json(ABI))
-
-
-protocol_group.add_command(contract_abi)
