@@ -1,33 +1,28 @@
-"""
-The `token` command group.
-"""
-
 from typing import Optional
 
-from click import ClickException, argument, command, group
+from click import ClickException, argument, group
 from web3 import Web3
+from web3.exceptions import ContractLogicError
+
+from autonity_cli.auth import validate_authenticator_account
 
 from ..denominations import format_quantity
 from ..erc20 import ERC20
 from ..options import (
-    from_option,
-    keyfile_option,
+    authentication_options,
+    from_options,
     newton_or_token_option,
     rpc_endpoint_option,
     tx_aux_options,
 )
 from ..utils import (
     create_contract_tx_from_args,
-    from_address_from_argument,
     newton_or_token_to_address_require,
     parse_token_value_representation,
     to_json,
     web3_from_endpoint_arg,
 )
 
-# pylint: disable=too-many-locals
-# pylint: disable=too-many-arguments
-
 
 @group(name="token")
 def token_group() -> None:
@@ -36,52 +31,52 @@ def token_group() -> None:
     """
 
 
-@command()
+@token_group.command()
 @rpc_endpoint_option
 @newton_or_token_option
 def name(rpc_endpoint: Optional[str], ntn: bool, token: Optional[str]) -> None:
     """
-    Returns the token name (if available).
+    The token's name (if available).
     """
 
     token_addresss = newton_or_token_to_address_require(ntn, token)
     w3 = web3_from_endpoint_arg(None, rpc_endpoint)
     erc = ERC20(w3, token_addresss)
-    token_name = erc.name()
-    if token_name is None:
-        raise ValueError("Token does not implement the name call")
+    try:
+        token_name = erc.name()
+    except ContractLogicError as exc:
+        raise ClickException(
+            "Token does not implement the ERC20 `name` function"
+        ) from exc
     print(token_name)
 
 
-token_group.add_command(name)
-
-
-@command()
+@token_group.command()
 @rpc_endpoint_option
 @newton_or_token_option
 def symbol(rpc_endpoint: Optional[str], ntn: bool, token: Optional[str]) -> None:
     """
-    Returns the token symbol (if available).
+    The token's symbol (if available).
     """
 
     token_addresss = newton_or_token_to_address_require(ntn, token)
     w3 = web3_from_endpoint_arg(None, rpc_endpoint)
     erc = ERC20(w3, token_addresss)
-    token_symbol = erc.symbol()
-    if token_symbol is None:
-        raise ClickException("Token does not implement the symbol call")
+    try:
+        token_symbol = erc.symbol()
+    except ContractLogicError as exc:
+        raise ClickException(
+            "Token does not implement the ERC20 `symbol` function"
+        ) from exc
     print(token_symbol)
 
 
-token_group.add_command(symbol)
-
-
-@command()
+@token_group.command()
 @rpc_endpoint_option
 @newton_or_token_option
 def decimals(rpc_endpoint: Optional[str], ntn: bool, token: Optional[str]) -> None:
     """
-    Returns the number of decimals used in the token balances.
+    The number of decimals used in the token balances.
     """
 
     token_addresss = newton_or_token_to_address_require(ntn, token)
@@ -90,10 +85,7 @@ def decimals(rpc_endpoint: Optional[str], ntn: bool, token: Optional[str]) -> No
     print(erc.decimals())
 
 
-token_group.add_command(decimals)
-
-
-@command()
+@token_group.command()
 @rpc_endpoint_option
 @newton_or_token_option
 def total_supply(rpc_endpoint: Optional[str], ntn: bool, token: Optional[str]) -> None:
@@ -109,29 +101,29 @@ def total_supply(rpc_endpoint: Optional[str], ntn: bool, token: Optional[str]) -
     print(format_quantity(token_total_supply, token_decimals))
 
 
-token_group.add_command(total_supply)
-
-
-@command()
+@token_group.command()
 @rpc_endpoint_option
 @newton_or_token_option
-@keyfile_option()
+@authentication_options()
 @argument("account_str", metavar="ACCOUNT", required=False)
 def balance_of(
     rpc_endpoint: Optional[str],
     ntn: bool,
     token: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     account_str: Optional[str],
 ) -> None:
     """
-    Returns the balance in tokens of ACCOUNT.
+    The balance of ACCOUNT in tokens.
 
     If ACCOUNT is not specified, the default keyfile is used.
     """
 
     token_addresss = newton_or_token_to_address_require(ntn, token)
-    account_addr = from_address_from_argument(account_str, keyfile)
+    account_addr = validate_authenticator_account(
+        account_str, keyfile=keyfile, trezor=trezor
+    )
 
     w3 = web3_from_endpoint_arg(None, rpc_endpoint)
     erc = ERC20(w3, token_addresss)
@@ -140,30 +132,27 @@ def balance_of(
     print(format_quantity(balance, token_decimals))
 
 
-token_group.add_command(balance_of)
-
-
-@command()
+@token_group.command()
 @rpc_endpoint_option
 @newton_or_token_option
-@keyfile_option()
-@from_option
+@from_options()
 @argument("owner")
 def allowance(
     rpc_endpoint: Optional[str],
     ntn: bool,
     token: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     owner: str,
 ) -> None:
     """
-    Returns the quantity in tokens that OWNER has granted the caller
-    (the "from" address) permission to spend.
+    The quantity of tokens that OWNER has granted the caller (`--from` address)
+    permission to spend.
     """
 
     token_addresss = newton_or_token_to_address_require(ntn, token)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     owner_addr = Web3.to_checksum_address(owner)
 
     w3 = web3_from_endpoint_arg(None, rpc_endpoint)
@@ -173,14 +162,10 @@ def allowance(
     print(format_quantity(token_allowance, token_decimals))
 
 
-token_group.add_command(allowance)
-
-
-@command()
+@token_group.command()
 @rpc_endpoint_option
 @newton_or_token_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("recipient_str", metavar="RECIPIENT")
 @argument("amount_str", metavar="AMOUNT")
@@ -189,6 +174,7 @@ def transfer(
     ntn: bool,
     token: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -207,7 +193,7 @@ def transfer(
     """
 
     token_addresss = newton_or_token_to_address_require(ntn, token)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     recipient_addr = Web3.to_checksum_address(recipient_str)
 
     w3 = web3_from_endpoint_arg(None, rpc_endpoint)
@@ -232,14 +218,10 @@ def transfer(
     print(to_json(tx))
 
 
-token_group.add_command(transfer)
-
-
-@command()
+@token_group.command()
 @rpc_endpoint_option
 @newton_or_token_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("spender_str", metavar="SPENDER")
 @argument("amount_str", metavar="AMOUNT")
@@ -249,6 +231,7 @@ def approve(
     token: Optional[str],
     keyfile: Optional[str],
     from_str: Optional[str],
+    trezor: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
     max_priority_fee_per_gas: Optional[str],
@@ -261,13 +244,13 @@ def approve(
 ) -> None:
     """
     Create a transaction granting SPENDER permission to spend
-    AMOUNT of tokens owned by `from_addr`.
+    AMOUNT of tokens owned by the caller (`--from` address).
 
     AMOUNT may be fractional if the token supports it.
     """
 
     token_addresss = newton_or_token_to_address_require(ntn, token)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     spender = Web3.to_checksum_address(spender_str)
 
     w3 = web3_from_endpoint_arg(None, rpc_endpoint)
@@ -292,14 +275,10 @@ def approve(
     print(to_json(tx))
 
 
-token_group.add_command(approve)
-
-
-@command()
+@token_group.command()
 @rpc_endpoint_option
 @newton_or_token_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("spender_str", metavar="SPENDER")
 @argument("recipient_str", metavar="RECIPIENT")
@@ -309,6 +288,7 @@ def transfer_from(
     ntn: bool,
     token: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -326,12 +306,12 @@ def transfer_from(
     to RECIPIENT.
 
     SPENDER must previously have granted the caller
-    (`from_addr`) permission to spend these tokens, via an `approve`
+    (`--from` address) permission to spend these tokens, via an `approve`
     transaction. AMOUNT can be fractional if the token supports it.
     """
 
     token_addresss = newton_or_token_to_address_require(ntn, token)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     spender = Web3.to_checksum_address(spender_str)
     recipient = Web3.to_checksum_address(recipient_str)
 
@@ -355,6 +335,3 @@ def transfer_from(
     )
 
     print(to_json(tx))
-
-
-token_group.add_command(transfer_from)
