@@ -1,23 +1,18 @@
-"""
-The `tx` command group.
-"""
-
 import asyncio
 import json
 from typing import Optional
 
-from click import ClickException, Path, argument, command, group, option
+from click import ClickException, Path, argument, group, option
 from eth_account.datastructures import SignedTransaction
-from eth_typing import HexStr
 from hexbytes import HexBytes
 from web3 import Web3
 
-from .account import signtx
+from autonity_cli.auth import validate_authenticator_account
+
 from ..erc20 import ERC20
 from ..logging import log
 from ..options import (
-    from_option,
-    keyfile_option,
+    from_options,
     newton_or_token_option,
     rpc_endpoint_option,
     tx_aux_options,
@@ -28,7 +23,6 @@ from ..utils import (
     create_contract_tx_from_args,
     create_tx_from_args,
     finalize_tx_from_args,
-    from_address_from_argument_optional,
     load_from_file_or_stdin,
     newton_or_token_to_address,
     parse_token_value_representation,
@@ -36,9 +30,7 @@ from ..utils import (
     validate_32byte_hash_string,
     web3_from_endpoint_arg,
 )
-
-# pylint: disable=too-many-locals
-# pylint: disable=too-many-arguments
+from .account import signtx
 
 
 @group(name="tx")
@@ -52,27 +44,22 @@ def tx_group() -> None:
 tx_group.add_command(signtx, name="sign")
 
 
-@command()
+@tx_group.command()
 @rpc_endpoint_option
 @newton_or_token_option
-@keyfile_option()
-@from_option
-@option("--to", "-t", "to_str", help="address to which tx is directed.")
+@from_options()
+@option("--to", "-t", "to_str", help="address to which the transaction is directed.")
 @tx_value_option(required=True)
 @tx_aux_options
 @option(
     "--data", "-d", help="compiled contract code OR method signature and parameters."
 )
-@option(
-    "--legacy",
-    is_flag=True,
-    help="if set, tx type is 0x0 (pre-EIP1559), otherwise type is 0x2.",
-)
 def make(
     rpc_endpoint: Optional[str],
     ntn: bool,
     token: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     to_str: Optional[str],
     gas: Optional[str],
@@ -84,10 +71,9 @@ def make(
     value: str,
     data: Optional[str],
     chain_id: Optional[int],
-    legacy: bool,
 ) -> None:
     """
-    Create a transaction given the parameters passed in.
+    Create a transaction from the parameters passed in.
     """
 
     # TODO: Add a flag which results in only unconnected Web3
@@ -98,16 +84,14 @@ def make(
     # Potentially used in multiple places, so avoid re-initializing.
     w3: Optional[Web3] = None
 
-    # If from_str is not set, take the address from a keyfile instead
-    # (if given)
-    from_addr = from_address_from_argument_optional(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     log(f"from_addr: {from_addr}")
 
     to_addr = Web3.to_checksum_address(to_str) if to_str else None
 
     if to_addr is None:
         raise ClickException(
-            "to-address must be specified.  (contract deployment not yet supported)"
+            "`--to` address must be specified (contract deployment not yet supported)"
         )
 
     token_addresss = newton_or_token_to_address(ntn, token)
@@ -120,7 +104,7 @@ def make(
 
     if token_addresss:
         if not from_addr:
-            raise ClickException("from address not given")
+            raise ClickException("`--from` address not specified")
 
         w3 = web3_from_endpoint_arg(w3, rpc_endpoint)
         erc = ERC20(w3, token_addresss)
@@ -140,10 +124,10 @@ def make(
 
     else:
         if not from_addr:
-            raise ClickException("from address not given")
+            raise ClickException("`--from` address not specified")
 
         if not value and not data:
-            raise ClickException("Empty tx (neither value or data given)")
+            raise ClickException("Empty transaction (neither value nor data specified)")
 
         tx, w3 = create_tx_from_args(
             w3,
@@ -165,26 +149,17 @@ def make(
 
     tx = finalize_tx_from_args(w3, rpc_endpoint, tx, from_addr)
 
-    # If the --legacy flag was given, explicitly set the type,
-    # otherwise have web3 determine it.
-
-    if legacy:
-        tx["type"] = HexStr("0x0")
-
     print(to_json(tx))
 
 
-tx_group.add_command(make)
-
-
-@command()
+@tx_group.command()
 @rpc_endpoint_option
 @argument("tx-file", type=Path())
 def send(rpc_endpoint: Optional[str], tx_file: str) -> None:
     """
-    Send raw transaction (as generated by signtx) contained in the given file.
+    Send raw transaction (as generated by `tx sign`) contained in the given file.
 
-    Use '-' to read from stdin instead of a file.
+    Use '-' to read from standard input instead of a file.
     Outputs the transaction hash if it is successfully sent.
     """
 
@@ -194,29 +169,26 @@ def send(rpc_endpoint: Optional[str], tx_file: str) -> None:
     print(Web3.to_hex(tx_hash))
 
 
-tx_group.add_command(send)
-
-
-@command()
+@tx_group.command()
 @rpc_endpoint_option
-@option("--quiet", "-q", is_flag=True, help="Do not dump the full transaction receipt.")
+@option("--quiet", "-q", is_flag=True, help="do not print the transaction receipt.")
 @option(
     "--timeout",
     "-t",
     type=float,
-    help="Wait up to some (non-whole) number of seconds.",
+    help="wait up to some (decimal) number of seconds.",
 )
 @argument("tx-hash", required=True)
 def wait(
     rpc_endpoint: Optional[str], quiet: bool, timeout: Optional[float], tx_hash: str
 ) -> None:
     """
-    Wait for a transaction with a specific hash, and dump the receipt.
+    Wait for a transaction with a specific hash, and print the receipt.
 
     The command will return exit code 0 if the transaction
     succeeded, or non-zero otherwise.
 
-    Timeouts also result in a non-zero exit code.
+    Timeout also results in a non-zero exit code.
     """
 
     hash_bytes = HexBytes(validate_32byte_hash_string(tx_hash))
@@ -231,9 +203,4 @@ def wait(
             raise ClickException("Transaction failed")
 
     except asyncio.TimeoutError:
-        raise ClickException(  # pylint: disable=raise-missing-from
-            "Tx {tx_hash} timed out"
-        )
-
-
-tx_group.add_command(wait)
+        raise ClickException(f"Transaction {tx_hash} timed out")
