@@ -82,7 +82,7 @@ def get_keyfile_password(password: Optional[str], keyfile: Optional[str] = None)
             password = getpass(
                 f"(consider using '{KEYFILE_PASSWORD_ENV_VAR}' env var).\n"
                 + "Enter passphrase "
-                + ("" if keyfile is None else f"for {keyfile} ")
+                + ("" if keyfile is None else f"for '{os.path.relpath(keyfile)}' ")
                 + "(or CTRL-d to exit): "
             )
 
@@ -102,7 +102,7 @@ def get_rpc_endpoint(endpoint: Optional[str]) -> str:
             endpoint = get_config_file().get("rpc_endpoint")
             if endpoint is None:
                 raise ClickException(
-                    f"No RPC endpoint given (use --rpc-endpoint, {WEB3_ENDPOINT_ENV_VAR}"
+                    f"No RPC endpoint given (use --rpc-endpoint, {WEB3_ENDPOINT_ENV_VAR} "
                     f"env var or {CONFIG_FILE_NAME})"
                 )
 
