@@ -13,12 +13,13 @@ from getpass import getpass
 from typing import Any, Dict, Mapping, Optional, Sequence, Tuple, TypeVar, Union, cast
 
 from autonity import Autonity
-from autonity.contracts import autonity
 from autonity.constants import AUTONITY_CONTRACT_ADDRESS
+from autonity.contracts import autonity
 from click import ClickException
 from eth_typing import ABI, ChecksumAddress
 from hexbytes import HexBytes
 from web3 import HTTPProvider, IPCProvider, LegacyWebSocketProvider, Web3
+from web3._utils.encoding import Web3JsonEncoder
 from web3.contract.contract import ContractFunction
 from web3.providers import BaseProvider
 from web3.types import (
@@ -27,23 +28,17 @@ from web3.types import (
     TxParams,
     Wei,
 )
-from web3._utils.encoding import Web3JsonEncoder
 
 from . import config
-from .constants import AutonDenoms, COMMISSION_RATE_PRECISION
+from .constants import COMMISSION_RATE_PRECISION, AutonDenoms
 from .denominations import NEWTON_DECIMALS
-from .keyfile import get_address_from_keyfile, load_keyfile
-from .logging import log
+from .keyfile import load_keyfile
 from .tx import (
     create_contract_function_transaction,
     create_transaction,
     finalize_transaction,
 )
 
-# pylint: disable=too-many-arguments
-# pylint: disable=too-many-locals
-
-
 # Intended to represent "value" types
 V = TypeVar("V")
 
@@ -107,48 +102,6 @@ def autonity_from_endpoint_arg(endpoint_arg: Optional[str]) -> autonity.Autonity
     return Autonity(web3_from_endpoint_arg(None, endpoint_arg))
 
 
-def from_address_from_argument_optional(
-    from_str: Optional[str], keyfile: Optional[str]
-) -> Optional[ChecksumAddress]:
-    """
-    Given an optional command line parameter, create an address,
-    falling back to the keyfile given in the config.  May be null if
-    neither is given.
-    """
-
-    # If from_str is not set, take the address from a keyfile instead
-    # (if given)
-    if from_str:
-        from_addr: Optional[ChecksumAddress] = Web3.to_checksum_address(from_str)
-    else:
-        log("no from-addr given.  attempting to extract from keyfile")
-        keyfile = config.get_keyfile_optional(keyfile)
-        if keyfile:
-            key_data = load_keyfile(keyfile)
-            from_addr = get_address_from_keyfile(key_data)
-            log(f"got keyfile: {keyfile}, address: {from_addr}")
-        else:
-            log("no keyfile.  empty from-addr")
-            from_addr = None
-    log(f"from_addr: {from_addr}")
-    return from_addr
-
-
-def from_address_from_argument(
-    from_str: Optional[str], keyfile: Optional[str]
-) -> ChecksumAddress:
-    """
-    Given an optional command line parameter, create an address,
-    falling back to the keyfile given in the config.  Throws a
-    ClickException if the address cannot be determined.
-    """
-    from_addr = from_address_from_argument_optional(from_str, keyfile)
-    if from_addr:
-        return from_addr
-
-    raise ClickException("from address or keyfile required")
-
-
 def create_tx_from_args(
     w3: Optional[Web3],
     rpc_endpoint: Optional[str],
@@ -386,7 +339,7 @@ def string_is_32byte_hash(hash_str: str) -> bool:
     try:
         int(hash_str, 16)
         return True
-    except Exception:  # pylint: disable=broad-except
+    except Exception:
         return False
 
 
@@ -412,18 +365,15 @@ def validate_block_identifier(block_id: Union[str, int]) -> BlockIdentifier:
     if isinstance(block_id, int):
         return block_id
 
-    if isinstance(block_id, str):
-        if block_id in ["latest", "earliest", "pending"]:
-            return cast(BlockIdentifier, block_id)
+    if block_id in ["latest", "earliest", "pending"]:
+        return cast(BlockIdentifier, block_id)
 
-        try:
-            return int(block_id)
-        except ValueError:
-            pass
+    try:
+        return int(block_id)
+    except ValueError:
+        pass
 
-        return HexBytes(block_id)
-
-    raise ClickException(f"failed parsing block identifier: {block_id}")
+    return HexBytes(block_id)
 
 
 def load_from_file_or_stdin(filename: str) -> str:
@@ -489,19 +439,15 @@ def newton_or_token_to_address_require(
     return token
 
 
-def prompt_for_new_password(show_password: bool) -> str:
+def prompt_for_new_password() -> str:
     """
     Prompt for a new password (with confirmation), optionally echoing
     to the console.
     """
     prompt = "Password for new account: "
     prompt_2 = "Confirm account password: "
-    if show_password:
-        password = input(prompt)
-        password_2 = input(prompt_2)
-    else:
-        password = getpass(prompt)
-        password_2 = getpass(prompt_2)
+    password = getpass(prompt)
+    password_2 = getpass(prompt_2)
 
     if password != password_2:
         raise ClickException("passwords do not match")
@@ -592,7 +538,7 @@ def parse_commission_rate(rate_str: str) -> int:
     try:
         rate_int = int(rate_str)
     except ValueError:
-        raise ClickException(  # pylint: disable=raise-missing-from
+        raise ClickException(
             f"Expected integer instead of {rate_str}.  See --help text."
         )
 
